#!/bin/sh

# check for filter string
while getopts "f:" opt; do
    case "$opt" in
        f)
            filter="$OPTARG"
            ;;
        h)
            echo "kakfilter -f <keys> [<file>]"
            echo "like kak -f, but supports commands"
            echo "accepts file over stdin or as argument"
            echo "applies <percent><keys> and prints buffer content to stdout"
            echo "kakrc is not loaded"
            ;;
        \?)
            echo "Invalid option"
            exit 1
            ;;
    esac
done

shift $((OPTIND - 1))

if [ -z "$filter" ]; then
    echo "No filter!"
    exit 1
fi

# create temp files
tempdir="$(mktemp -d kakpipe-XXXXXX)"
fifo="${tempdir}/fifo"
# prioritize arg if it exists
# use stdin otherwise
if [ -z "$1" ]; then
    input="$(mktemp -p ${tempdir} input.XXXXXX)"
    cat >"$input"
else
    input="$1"
fi

filter=$(echo $filter | sed "s/'/<quote>/g; s/\"/<dquote>/g; s/ /<space>/g")

# run kak as a filter
# write buffer to fifo before quitting
cmd="$(printf '%s\n' "exec <percent>; execute-keys '$filter' ; execute-keys <esc>; write $fifo; quit!")"
echo "$cmd"
kak -n -e "$cmd" "$input"
# print kak output to stdout
cat "$fifo"
# cleanup
rm -r "$tempdir"
